==> ndash-0.01.tar.gz: ndash-0.01/lib/Nagios/Status.pm <==
package Nagios::Status;

use Mouse;
use Nagios::Status::Service;
use IO::File;
use Carp;

has 'file'          => ( is => 'rw', isa => 'Str' );
has 'hosts'         => ( is => 'rw', isa => 'ArrayRef', auto_deref => 1 );
has 'host_services' => ( is => 'rw', isa => 'HashRef' );

sub BUILD {
  my ($self, $params) = @_;

  my $file = $params->{file} || '/var/nagios/status.dat';
  die "Cannot find groups file '$file'" unless -f $file;
  $self->file( $file );

  my %host_services = ();
  
  my $fh = IO::File->new( $file, 'r' )
    or die "Cannot open status file '$file': $!";
  my $section = '';
  my $line = 0;
  for (<$fh>) {
    $line++;
    next unless m/\S/;
    next if m/^#/;
    chomp;
    if ($section) {
      if (m/^\s*}\s*$/) {
        if ($section->{NAME} eq 'servicestatus') {
          my $host = $section->{host_name};
          $host_services{$host} ||= [];
          push @{ $host_services{$host} }, Nagios::Status::Service->new(
            description     => $section->{ service_description },
            state           => $section->{ current_state },
            plugin_output   => $section->{ plugin_output },
          );
        }
        $section = '';
      }
      elsif (m/(\w+)=(\S*)/) {
        my $value = defined $2 ? $2 : '';
        $section->{$1} = $value;
      }
      else {
        die "Cannot parse line '$_', section $section->{NAME}, line $line";
      }
    }
    else {
      if (m/^\s*(\w+)\s*{\s*$/) {
        $section = { NAME => $1 };
      }
    }
  }

  $self->host_services( \%host_services );
}

sub services {
  my ($self, $host) = @_;
  croak "Missing 'host' argument" unless $host;
  return () unless $self->host_services->{$host};
  return wantarray ? @{ $self->host_services->{$host} } : $self->host_services->{$host};
}

1;

=head1 NAME

Nagios::Status - class modelling service statuses from Nagios v3 status logs

=head1 SYNOPSIS

    use Nagios::Status;

    $ns = Nagios::Status->new;

    @hosts = $ns->hosts;

    @services = $ns->services( $host );


=head1 SEE ALSO

Nagios::Object, Nagios::StatusLog

=head1 ACKNOWLEDGEMENTS

=head1 COPYRIGHT AND LICENCE

Copyright 2009 Gavin Carr.

This software is free software, licensed under the same terms as Perl itself.

=cut

==> ndash-0.02.tar.gz: ndash-0.02/lib/Nagios/Status.pm <==
package Nagios::Status;

use Mouse;
use Nagios::Status::Service;
use IO::File;
use Carp;

has 'file'          => ( is => 'rw', isa => 'Str' );
#has 'hosts'         => ( is => 'rw', isa => 'ArrayRef', auto_deref => 1 );
has 'host_services' => ( is => 'rw', isa => 'HashRef' );

sub BUILD {
  my ($self, $params) = @_;

  my $file = $params->{file} || '/var/nagios/status.dat';
  die "Cannot find groups file '$file'" unless -f $file;
  $self->file( $file );

  my %host_services = ();
  
  my $fh = IO::File->new( $file, 'r' )
    or die "Cannot open status file '$file': $!";
  my $section = '';
  my $line = 0;
  for (<$fh>) {
    $line++;
    next unless m/\S/;
    next if m/^#/;
    chomp;
    if ($section) {
      if (m/^\s*}\s*$/) {
        if ($section->{NAME} eq 'servicestatus') {
          my $host = $section->{host_name};
          $host_services{$host} ||= [];
          push @{ $host_services{$host} }, Nagios::Status::Service->new(
            description     => $section->{ service_description },
            state           => $section->{ current_state },
            plugin_output   => $section->{ plugin_output },
          );
        }
        $section = '';
      }
      elsif (m/(\w+)=(\S*)/) {
        my $value = defined $2 ? $2 : '';
        $section->{$1} = $value;
      }
      else {
        die "Cannot parse line '$_', section $section->{NAME}, line $line";
      }
    }
    else {
      if (m/^\s*(\w+)\s*{\s*$/) {
        $section = { NAME => $1 };
      }
    }
  }

  $self->host_services( \%host_services );
}

sub services {
  my ($self, $host) = @_;
  croak "Missing 'host' argument" unless $host;
  return () unless $self->host_services->{$host};
  return wantarray ? @{ $self->host_services->{$host} } : $self->host_services->{$host};
}

1;

=head1 NAME

Nagios::Status - class modelling service statuses from Nagios v3 status logs

=head1 SYNOPSIS

    use Nagios::Status;

    $ns = Nagios::Status->new;

    @services = $ns->services( $host );


=head1 SEE ALSO

Nagios::Object, Nagios::StatusLog

=head1 COPYRIGHT AND LICENCE

Copyright 2009 Gavin Carr L< <<gavin@openfusion.com.au>> >.

This software is free software, licensed under the same terms as Perl itself.

=cut
